import { ChatService } from '../../service/platform/chat.service';
import { CVO_chat, FVO_chat, QVO_chat, UVAO_chat } from '../../interface/platform/chat.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { omit, pick } from 'lodash';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
@ApiTags(['聊天记录'])
@Controller('/chat')
export class ChatController implements BaseController {
  @Inject()
  service: ChatService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_chat })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_chat })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_chat(data);
    return result;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_chat })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_chat(dbData);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_chat })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
