import { CollectionService } from '../../service/platform/collection.service';
import { CDTO_collection, CVO_collection, FVO_collection, QVO_collection, UVAO_collection } from '../../interface/platform/collection.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
const namePrefix = '收藏';
@ApiTags(['收藏'])
@Controller('/collection', { tagName: namePrefix })
export class CollectionController implements BaseController {
  @Inject()
  service: CollectionService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_collection })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    const list = [];
    for (const i of data) {
      list.push(await this.service.fillScource(i));
    }
    return { data: list, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_collection })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_collection(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_collection })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_collection(dbData);
    return result;
  }

  @Post('/cancel')
  @Validate()
  @ApiResponse({ type: CVO_collection })
  async cancel(@Body() data: CDTO_collection) {
    await this.service.cancel(data);
    return 'ok';
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_collection })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
