import { DemandService } from '../../service/platform/demand.service';
import { CVO_demand, FVO_demand, QVO_demand, UVAO_demand } from '../../interface/platform/demand.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { Context } from '@midwayjs/koa';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '需求';
@ApiTags(['需求'])
@Controller('/demand', { tagName: namePrefix })
export class DemandController implements BaseController {
  @Inject()
  service: DemandService;
  @Inject()
  ctx: Context;
  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_demand })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_demand })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_demand(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_demand })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_demand(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_demand })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others, {
      field: this.service.Opera.In,
      name: this.service.Opera.Like,
      company: this.service.Opera.Like,
      industry: this.service.Opera.In,
      area: this.service.Opera.Json,
      tags: this.service.Opera.Json,
    });
    for (let i of data) {
      i = await this.serviceUtil.fillOnwer(i);
    }
    return { data, total };
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_demand })
  async detail(@Param('id') id: string) {
    let data = await this.service.fetch({ id });
    data = await this.serviceUtil.fillOnwer(data);
    data = await this.serviceUtil.fillCollection(data, 'demand');
    return data;
  }
}
