import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { FootplateService } from '../../service/platform/footplate.service';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { QVO_footplate, FVO_footplate, CVO_footplate, UVAO_footplate } from '../../interface/platform/footplate.interface';
import { Context } from '@midwayjs/koa';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '中试平台';
@ApiTags(['中试平台'])
@Controller('/footplate', { tagName: namePrefix })
export class FootplateController implements BaseController {
  @Inject()
  service: FootplateService;

  @Inject()
  ctx: Context;

  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_footplate })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    return { data, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_footplate })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_footplate(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_footplate })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_footplate(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_footplate })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_footplate })
  async detail(@Param('id') id: string) {
    let data = await this.service.fetch({ id });
    data = await this.serviceUtil.fillCollection(data, 'footplate');
    return data;
  }
}
