import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { CVO_friend, FVO_friend, QVO_friend, UVAO_friend } from '../../interface/platform/friend.interface';
import { FriendService } from '../../service/platform/friend.service';
import { Context } from '@midwayjs/koa';
const namePrefix = '合作伙伴';
@ApiTags(['合作伙伴'])
@Controller('/friend', { tagName: namePrefix })
export class FriendController implements BaseController {
  @Inject()
  service: FriendService;

  @Inject()
  ctx: Context;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_friend })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/list')
  async list() {
    const list = await this.service.list(this.ctx.query);
    const data = list.data;
    const total = list.total;
    return { data, total };
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_friend })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_friend(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_friend })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_friend(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_friend })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
