import { MatchPathService } from '../../service/platform/matchPath.service';
import { CVO_matchPath, FVO_matchPath, QVO_matchPath, UVAO_matchPath } from '../../interface/platform/matchPath.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '赛事流程';
@ApiTags(['赛事流程'])
@Controller('/matchPath', { tagName: namePrefix })
export class MatchPathController implements BaseController {
  @Inject()
  service: MatchPathService;
  @Inject()
  serviceUtil: ServiceUtilService;
  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_matchPath })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others: any = pick(query, ['skip', 'limit']);
    others.order = { order_num: 'ASC' };
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_matchPath })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_matchPath(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_matchPath })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_matchPath(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_matchPath })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
