import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { ScoreService } from '../../service/platform/score.service';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { QVO_score, FVO_score, CVO_score, UVAO_score } from '../../interface/platform/score.interface';
import { Context } from '@midwayjs/koa';
const namePrefix = '分数';
@ApiTags(['分数'])
@Controller('/score', { tagName: namePrefix })
export class ScoreController implements BaseController {
  @Inject()
  service: ScoreService;
  @Inject()
  ctx: Context;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_score })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_score })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_score(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_score })
  async create(@Body() data: object) {
    await this.service.createExamine(data);
    const dbData = await this.service.create(data);
    const result = new CVO_score(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_score })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/list')
  async list(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    const list = [];
    for (const i of data) {
      list.push(await this.service.fillName(i));
    }
    return { data: list, total };
  }
}
