import { SectorService } from '../../service/platform/sector.service';
import { CVO_sector, FVO_sector, QVO_sector, UVAO_sector } from '../../interface/platform/sector.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { omit, pick } from 'lodash';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
const namePrefix = '行业';
@ApiTags(['行业'])
@Controller('/sector', { tagName: namePrefix })
export class SectorController implements BaseController {
  @Inject()
  service: SectorService;

  @Inject()
  ctx: Context;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_sector })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others: any = pick(query, ['skip', 'limit']);
    others.order = { sort: 'ASC' };
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_sector })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_sector(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_sector })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_sector(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_sector })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
