import { SignService } from '../../service/platform/sign.service';
import { CVO_sign, FVO_sign, QVO_sign, UVAO_sign } from '../../interface/platform/sign.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { BaseController } from '../../frame/BaseController';
import { Context } from '@midwayjs/koa';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '赛事报名表';

@ApiTags(['赛事报名表'])
@Controller('/sign', { tagName: namePrefix })
export class SignController implements BaseController {
  @Inject()
  service: SignService;

  @Inject()
  ctx: Context;

  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_sign })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_sign })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_sign(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_sign })
  async create(@Body() data: any) {
    await this.service.createExamine(data);
    if (data.type && data.type === '0') data.status = '1';
    const dbData = await this.service.create(data);
    const result = new CVO_sign(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_sign })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    await this.service.updateExamine(data);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
  @Get('/sign')
  async sign(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const { data, total } = await this.service.query(qobj, others);
    const list = [];
    for (const i of data) {
      list.push(await this.serviceUtil.fillMatchInfo(i));
    }
    return { data: list, total };
  }
}
