import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { CVO_region, FVO_region, QVO_region, UVAO_region } from '../../interface/system/region.interface';
import { RegionService } from '../../service/system/region.service';
import { Context } from '@midwayjs/koa';
const namePrefix = '地区';
@ApiTags(['地区'])
@Controller('/region', { tagName: namePrefix })
export class RegionController implements BaseController {
  controllerCode = 'system_region';
  @Inject()
  service: RegionService;

  @Inject()
  ctx: Context;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_region })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_region })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_region(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_region })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_region(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_region })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }

  @Get('/area')
  async area() {
    const list = await this.service.area(this.ctx.query);
    const data = list.data;
    const total = list.total;
    return { data, total };
  }

  @Get('/list')
  async list() {
    const list = await this.service.list(this.ctx.query);
    const data = list.data;
    const total = list.total;
    return { data, total };
  }
}
