import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { TagsService } from '../../service/system/tags.service';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { BaseController } from '../../frame/BaseController';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { QVO_tags, FVO_tags, CVO_tags, UVAO_tags } from '../../interface/system/tags.interface';
const namePrefix = '标签';

@ApiTags(['标签'])
@Controller('/tags', { tagName: namePrefix })
export class TagsController implements BaseController {
  controllerCode = 'system_tags';
  @Inject()
  service: TagsService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_tags })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others: any = pick(query, ['skip', 'limit']);
    others.order = { num: 'ASC' };
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_tags })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_tags(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_tags })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_tags(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_tags })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
