import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { BaseController } from '../../frame/BaseController';
import { UserMenusService } from '../../service/system/userMenus.service';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { CVO_menus, FVO_menus, QVO_menus, UVAO_menus } from '../../interface/system/menus.interface';

const namePrefix = '用户目录';
@ApiTags(['用户目录'])
@Controller('/userMenus', { tagName: namePrefix })
export class UserMenusController implements BaseController {
  controllerCode = 'system_userMenus';
  @Inject()
  service: UserMenusService;
  
  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_menus })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, { ...others, order: { order_num: 'ASC' } });
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_menus })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_menus(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_menus })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_menus(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_menus })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
