import { Body, Controller, Get, Inject, Post } from '@midwayjs/core';
import { InitSystemDataService } from '../service/initData/initSystemData.service';
import { DataDealService } from '../service/initData/dataDeal.service';
import { NeedAdminMiddleware } from '../middleware/needAdmin.middleware';
import { ESService } from '../service/elasticsearch/es.service';
import { DbBackupService } from '../service/dbBackup.service';

/**所有的操作都不需要await.目前是 下达一个无监控的任务进程 */
@Controller('/system/func', { middleware: [NeedAdminMiddleware] })
export class SystemFuncController {
  @Inject()
  InitSystemData: InitSystemDataService;
  @Inject()
  DataDeal: DataDealService;
  @Inject()
  es: ESService;
  @Inject()
  dbBackUp: DbBackupService;

  @Post('/dbRestore', { routerName: `还原数据库` })
  async dbRestore(@Body('path') path: string) {
    this.dbBackUp.restore(path);
    return 'ok'
  }

  @Get('/dbBackUp', { routerName: `备份数据库` })
  async dbBackUpIntoJson() {
    await this.dbBackUp.getTables();
  }

  @Get('/')
  async index() {
    console.log('in');
  }

  @Get('/initES', { routerName: `初始化es索引及数据` })
  async initES() {
    this.es.initES();
  }

  /**修正导入的数据为 richText的数据 */
  @Get('/cid', { routerName: `修正数据` })
  async correctImportData() {
    this.DataDeal.correctImportData();
  }

  /**初始化后台目录 */
  @Get('/ism', { routerName: `初始化后台目录` })
  async initSystemMenus() {
    this.InitSystemData.initSystemMenus();
  }

  /**初始化用户目录 */
  @Get('/ium', { routerName: `初始化用户目录` })
  async initUserMenus() {
    this.InitSystemData.initUserMenus();
  }
  /**初始化角色数据 */
  @Get('/ir', { routerName: `初始化角色数据` })
  async initRoleData() {
    this.InitSystemData.initRoleData();
  }
  /**初始化角色的菜单(不包含管理员) */
  @Get('/irm', { routerName: `初始化角色菜单` })
  async initRoleMenus() {
    this.InitSystemData.initRoleMenus();
  }
}
