import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { Context } from '@midwayjs/koa';
import { CompanyYearService } from '../../service/users/companyYear.service';
import { QVO_companyYear, FVO_companyYear, CVO_companyYear, UVAO_companyYear } from '../../interface/users/companyYear.interface';
import { ServiceUtilService } from '../../service/serviceUtil.service';
const namePrefix = '企业年度信息';
@ApiTags(['企业年度信息'])
@Controller('/companyYear', { tagName: namePrefix })
export class CompanyYearController implements BaseController {
  @Inject()
  service: CompanyYearService;
  @Inject()
  ctx: Context;

  @Inject()
  serviceUtil: ServiceUtilService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_companyYear })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const result = await this.service.query(qobj, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_companyYear })
  async fetch(@Param('id') id: number) {
    const data = await this.service.fetch({ id });
    const result = new FVO_companyYear(data);
    return result;
  }

  @Post('/', { routerName: `创建${namePrefix}` })
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_companyYear })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_companyYear(dbData);
    return result;
  }

  @Post('/:id', { routerName: `修改${namePrefix}` })
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_companyYear })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id', { routerName: `删除${namePrefix}` })
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
