import { Provide } from '@midwayjs/core';
import { InjectEntityModel } from '@midwayjs/typeorm';
import { Repository } from 'typeorm';
import { BaseServiceV2 } from '../../frame/BaseServiceV2';
import { MatchRegistration } from '../../entity/match/matchRegistration.entity';
import { get, orderBy } from 'lodash';
@Provide()
export class MatchRegistrationService extends BaseServiceV2 {
  @InjectEntityModel(MatchRegistration)
  model: Repository<MatchRegistration>;

  /**
   * 根据赛事id,查出决赛的名单,按人为规定最终排名参与的排序
   * @param match_id 赛事id
   */
  async getArrangeList(match_id) {
    const { data: list, total } = await this.query({ match_id, order: { final_score: "DESC" } });
    /**有指定最终排名的数据 */
    const orderList = orderBy(list.filter(f => f.last_order_no), ['last_order_no'], ['asc'])
    /**没有指定最终排名的数据 */
    const freeOrderList = orderBy(list.filter(f => !f.last_order_no), ['final_score'], ['desc'])
    for (const i of orderList) {
      const no = get(i, 'last_order_no')
      if (!no || no <= 0) continue
      const index = no - 1
      freeOrderList.splice(index, 0, i)
    }
    for (let i = 0; i < freeOrderList.length; i++) {
      const item = freeOrderList[i];
      item.show_order = i + 1
    }
    return freeOrderList;
  }
}
