import { Provide } from '@midwayjs/core';
import { InjectEntityModel } from '@midwayjs/typeorm';
import { Equal, Repository } from 'typeorm';
import { User } from '../../entity/system/user.entity';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { BaseServiceV2 } from '../../frame/BaseServiceV2';
import { get } from 'lodash';
@Provide()
export class UserService extends BaseServiceV2 {
  @InjectEntityModel(User)
  model: Repository<User>;
  getQueryColumnsOpera() {
    return { nick_name: this.Opera.Like };
  }
  async createBefore(data) {
    const { role } = data;
    if (role && role.length > 1) data.status = '0';
    else data.status = '1';
    return data;
  }
  async checkEmail(data) {
    const email = get(data, 'email');
    if (!email) return;
    const builder = await this.model.createQueryBuilder().where('"email" =:value', { value: email });
    // 如果有id,则视为修改,查非自己的email是否存在
    const id = get(data, 'id');
    if (id) builder.andWhere('"id" != :id', { id });
    const num = await builder.getCount();
    if (num <= 0) return true;
    // 邮箱存在
    throw new ServiceError(ErrorCode.EMAIL_IS_EXISTS);
  }

  async checkPhone(data) {
    const phone = get(data, 'phone');
    if (!phone) return;
    const builder = await this.model.createQueryBuilder().where('"phone" =:value', { value: phone });
    // 如果有id,则视为修改,查非自己的phone是否存在
    const id = get(data, 'id');
    if (id) builder.andWhere('"id" != :id', { id });
    const num = await builder.getCount();
    if (num <= 0) return true;
    // 手机号存在
    throw new ServiceError(ErrorCode.PHONE_IS_EXISTS);
  }

  // 用户账号，微信唯一标识检查
  async createExamine(data) {
    const { openid, account } = data;
    let result;
    if (account) {
      result = await this.model.findOne({ where: { account: Equal(account) } });
      if (result) throw new ServiceError(ErrorCode.DUPLICATE_USERS_ACCOUNT);
    }
    if (openid) {
      result = await this.model.findOne({ where: { openid: Equal(openid) } });
      if (result) throw new ServiceError(ErrorCode.DUPLICATE_USERS_OPENID);
    }
  }

  // 修改后判断是否有身份 如何审核通过将有身份的数据也审核成功
  async updateAfter(id, body) {
    const arr = await this.detail(id);
    const { status } = body;
    if (status === '1' && arr.role_type !== 'User') {
      // const model = GetModel(upperFirst(arr.role_type));
      // 需要明确修改的指代内容了:如果不是User，那还得说明是哪个 users下面的所有都是，都得引入进来处理
      // const user = await model.findOne({ where: { user: id } });
      // await model.update({ where: { id: user.id } }, { status: '1' });
    }
  }

  async detail(id) {
    const arr = await this.model.findOne({ where: { id: Equal(id) } });
    let role_type;
    if (arr && arr.role && arr.role.length > 1) role_type = arr.role[1];
    else role_type = arr.role[0];
    return { ...arr, role_type };
  }
}
